-- LOO: Lua Object Oriented
-- Author: Michael Carius <michael.a.carius@gmail.com>

module(..., package.seeall)

-- Python-style classes
-- (sorta)
function class(class)
    -- Safely override __index
    -- This lets us use the class as the instance metatable, which lets users
    -- transparently define metamethods for their objects
    if class.__index then
        local oldIndex = class.__index
        if type(oldIndex) == "table" then
            class.__index = function(class, key)
                return oldIndex[key] or class[key]
            end
        elseif type(oldIndex) == "function" then
            class.__index = function(class, key)
                return oldIndex(class, key) or class[key]
            end
        end
    else
        class.__index = class
    end

    local metaclass = {
        __call = function(class, ...)
            local instance = setmetatable({}, class)
            if class.initialize then
                instance:initialize(...)
            end
            return instance
        end,
    }

    return setmetatable(class, metaclass)
end
